#include "Database.h"

#include <Base\Containers\range_for_std_containers.hpp>
#include <Base\IO\FileUtil.hpp>
#include <Base\IO\MemoryReadStream.h>
#include <Base\IO\ReadStreamBinLex.h>
#include <Base\IO\ReadStreamBinLex_stdvector.h>
#include <Base\Math\math_strfmt.h>
#include <Base\String\Formatting.h>

namespace hyplex::drakan
{
	static const bool kEnableDump = true;

	void Animation::consumeData(const Entry& entry, BasicBufferedLexState& bin)
	{
		// 01 05 Animation info
		// 02 05 Animation frames
		// 03 05 Animation frame index ranges

		if (entry.type == 1)
		{
			name = trimNul(readString(bin, read<uint16_t>(bin)));

			const struct Header
			{
				float duration = 0;
				uint32_t originalNumFrames = 0;
				uint32_t numFrames = 0;
				uint32_t unk0 = 0;
				uint32_t modelNumNodes = 0;
				uint32_t modelNumChannels = 0;
				uint32_t unk1 = 0; // Non-Looping flag?
				uint32_t refCount = 0;
				float rotationThreshold = 0;
				float scaleThreshold = 0;
				float translationThreshold = 0; // Probably.
			} header = read<Header>(bin);

			duration = header.duration;
			originalNumFrames = header.originalNumFrames;
			numFrames = header.numFrames;
			unk0 = header.unk0;
			modelNumNodes = header.modelNumNodes;
			modelNumChannels = header.modelNumChannels;
			unk1 = header.unk1;
			refCount = header.refCount;
			rotationThreshold = header.rotationThreshold;
			scaleThreshold = header.scaleThreshold;
			translationThreshold = header.translationThreshold;
		}
		else if (entry.type == 2)
			readArray(bin, frames, read<uint16_t>(bin));
		else if (entry.type == 3)
			readArray(bin, nodeFramesIndexRanges, read<uint16_t>(bin));
	}
	ENodeType Animation::getType() const noexcept
	{
		return ENodeType::ANIMATION;
	}
	void Animation::dumpNode(const NodeDumpArg& arg) const
	{
		if (!kEnableDump)
			return;

		String str;

		printfmt(str, "id %u\n", id);
		printfmt(str, "name %s\n", name);
		printfmt(str, "duration %f\n", duration);
		printfmt(str, "originalNumFrames %u\n", originalNumFrames);
		printfmt(str, "numFrames %u\n", numFrames);
		printfmt(str, "unk0 %08x\n", unk0);
		printfmt(str, "modelNumNodes %u\n", modelNumNodes);
		printfmt(str, "modelNumChannels %u\n", modelNumChannels);
		printfmt(str, "unk1 %08x\n", unk1);
		printfmt(str, "refCount %u\n", refCount);
		printfmt(str, "rotationThreshold %f\n", rotationThreshold);
		printfmt(str, "scaleThreshold %f\n", scaleThreshold);
		printfmt(str, "translationThreshold %f\n", translationThreshold);

		for (unsigned int i = 0; i < nodeFramesIndexRanges.size(); ++i)
		{
			const NodeFramesIndexRange ir = nodeFramesIndexRanges[i];
			const array_view<const Frame> nodeFrames = asArrayView(frames).slice_n(ir.firstFrameI, ir.numFrames);

			printfmt(str, "Node %u\n", i);
			for (unsigned int j = 0; j < nodeFrames.size(); ++j)
				printfmt(str, "\t %.4f % .6f\n", nodeFrames[j].t, nodeFrames[j].xform);
			str += '\n';
		}

		FileUnbufferedWriteStream(sprintfmt("%sAnimations\\Anim%u %s.txt", arg.dir, id, name), EFileWriteOpenMode::RECREATE, true)
			.write(asRawUChars(asArrayView(str)));
	}
}