#include "FilesystemPathUtil.h"

namespace hyplex
{
	namespace fs = std::experimental::filesystem;

	fs::path makeRelativePath(const fs::path& path, const fs::path& base)
	{
		// http://en.cppreference.com/w/cpp/filesystem/path/lexically_normal

		if (path.root_name() != base.root_name() || path.is_absolute() != base.is_absolute() || (!path.has_root_directory() && base.has_root_directory()))
			return fs::path();

		const auto[a, b] = std::mismatch(path.begin(), path.end(), base.begin(), base.end());
		if (a == path.end() && b == base.end())
			return L".";

		const size_t numBaseDotDot = std::count_if(b, base.end(), [](const auto& x) { return x == L".."; });
		const size_t numBaseNames = std::count_if(b, base.end(), [](const auto& x) { return x != L".." && x != L"."; });
		if (numBaseDotDot > numBaseNames)
			return fs::path();

		fs::path rel;
		for (unsigned int i = 0; i < numBaseNames - numBaseDotDot; ++i)
			rel /= L"..";
		for (auto it = a; it != path.end(); ++it)
			rel /= *it;
		return rel;
	}
}