#pragma once

#include <Base\Containers\array_view.hpp>
#include <Base\Math\vec2.h>

#include <memory>
#include <vector>

namespace hyplex::gfx
{
	enum class EPixelFormat: uint16_t
	{
		DXT1,
		DXT3,
		DXT5,
		U8RGBA,
		U8RGB,
		U8BGR,
		U8L,
		U8LA,
		U8A,
	};

	bool isCompressedBlockFormat(EPixelFormat) noexcept;
	unsigned int getBytesPerPixel(EPixelFormat) noexcept;

	class IImage
	{
	public:
		struct Info
		{
			EPixelFormat pixelFmt;
			uint16_t numMipmaps;
			uivec2 dimensions;
			unsigned int bytesPerRow;
		};

		static const std::shared_ptr<const IImage> kDefaultWhitePixelRGBImage;

		virtual EPixelFormat getPixelFormat() const = 0;
		virtual uivec2 getDimensions() const = 0;
		virtual unsigned int getBytesPerRow() const = 0;
		virtual unsigned int getNumMipMaps() const = 0;
		virtual RawConstMemoryRange getRawData() const = 0;

		Info getInfo() const
		{
			return { getPixelFormat(), (uint16_t)getNumMipMaps(), getDimensions(), getBytesPerRow() };
		}

		virtual ~IImage() = default;
	};

	std::vector<char> makePNG(const IImage& img);
}