#pragma once

#include <Base\Containers\index_range.hpp>
#include <Base\String\String.hpp>

#include <map>
#include <vector>

namespace hyplex::drakan
{
	struct Class;

	using ClassCategoryLookup = std::map<unsigned int, String>;
	using ClassLookup = std::map<unsigned int, Class>;

	enum class EDataType: uint16_t
	{
		NONE,
		INT,
		FLOAT,
		OBJECT,
		MODEL,
		SOUND,
		UNK6,
		CHOICE, // By default, this is a boolean, but an enum string list may be supplied.
		ANIM_CHANNEL,
		ANIMATION,
		STRING,
		SEQUENCE,
		LANDSCAPE_LAYER,
		UNK13,
		TEXTURE,
		COLOUR,
	};

	struct DataType
	{
		EDataType elementType = EDataType::NONE;
		bool isList = false;
		bool isObjectOfClassCategory = false;

		DataType() = default;
		explicit DataType(uint32_t rawTypeInfo) noexcept;
	};

	struct Class
	{
		struct PropertySection
		{
			String name;
			index_range propsIR;
		};

		struct Property
		{
			String name;

			DataType type;

			union
			{
				unsigned int objClassCategoryId = 0;
				unsigned int objClassId;
			};
		
			union
			{
				int32_t i32;
				float f32;
				std::array<uint8_t, 4> argb;
			} defaultValue;

			
		};

		unsigned int id = 0;
		String name;
		unsigned int categoryId = 0;
		uint32_t unk08 = 0;
		uint32_t unk0C = 0;
		uint32_t unk14 = 0;

		std::vector<PropertySection> propertySections;
		std::vector<Property> props;

		String toString(const ClassCategoryLookup& classCats, const ClassLookup& classes) const;
	};

	void extractRFLResources(string_view inputRFL, string_view outputDir);

	ClassCategoryLookup loadRFLClassCategories(RawConstMemoryRange blob);
	std::map<unsigned int, Class> loadRFLClasses(RawConstMemoryRange blob);
}